package es.verdnatura.presentation.view.feature.boxPicking

import android.view.View.VISIBLE
import android.view.inputmethod.EditorInfo
import es.verdnatura.R
import es.verdnatura.databinding.FragmentGeneralBlackBinding
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.ExpeditionPrintOut
import es.verdnatura.presentation.common.OnCollectionSelectedListener
import es.verdnatura.presentation.view.component.CustomDialogInput

class BoxPickingFragment(var title: String) :
    BaseFragment<FragmentGeneralBlackBinding, BoxPickingViewModel>(BoxPickingViewModel::class) {
    private var onCollectionSelectedListener: OnCollectionSelectedListener? = null
    private var isBoxScanned: Boolean = false
    private lateinit var customDialogInput: CustomDialogInput
    override fun getLayoutId(): Int = R.layout.fragment_general_black

    companion object {
        fun newInstance(title: String) = BoxPickingFragment(title = title)
    }

    override fun init() {

        binding.mainToolbar.toolbarTitle.text = title
        setEvents()
        customDialogInput = CustomDialogInput(requireContext())
        super.init()
    }

    private fun setEvents() {
        binding.scanInput.visibility = VISIBLE
        binding.mainToolbar.backButton.setOnClickListener {
            ma.onMyBackPressed()
        }
        binding.scanInput.hint = getString(R.string.scanLabelExpedition)
        binding.scanInput.requestFocus()
        binding.scanInput.setOnEditorActionListener { _, actionId, _ ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE) {

                if (!binding.scanInput.text.isNullOrEmpty()) {
                    try {
                        showScanBarcode(binding.scanInput.text.toString().toLong())
                    }catch (ex:Exception){
                        ma.messageWithSound(message =getString(R.string.itemNotValid), isError = true, isPlayed = true, isToasted = false)
                    }


                }
                binding.scanInput.setText("")
                ma.hideKeyboard(binding.scanInput)
                return@setOnEditorActionListener true
            }
            false
        }
    }

    private fun showScanBarcode(expeditionFk: Long) {
        try {

            customDialogInput.setTitle(getString(R.string.scanItem))
                .setOkButton(getString(R.string.accept)) {
                    if (customDialogInput.getValue().isNotEmpty()) {
                        viewModel.isBoxPickingInPrintOut(
                            expeditionFk = expeditionFk,
                            customDialogInput.getValue()
                        )
                    }
                    customDialogInput.dismiss()
                }

                .setKoButton(getString(R.string.cancel)) {
                    customDialogInput.setValue("")
                    customDialogInput.dismiss()

                }.setValue("").show()




            customDialogInput.getEditText().setOnEditorActionListener { _, actionId, _ ->
                if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {

                    if (customDialogInput.getValue().isNotEmpty()) {
                        viewModel.isBoxPickingInPrintOut(
                            expeditionFk = expeditionFk,
                            customDialogInput.getValue()
                        )
                    }
                    customDialogInput.dismiss()
                    binding.scanInput.requestFocus()
                    return@setOnEditorActionListener true
                }
                false
            }

        } catch (ex: Exception) {
            getString(R.string.errorInput)
        }
    }

    override fun observeViewModel() {
        with(viewModel) {
            /*isBoxPickingInPrintOut.observe(viewLifecycleOwner) {

                    ma.messageWithSound(
                        message = getString(R.string.errorInput),
                        isError = !it,
                        isPlayed = true,
                        isToasted = null

                    )
                binding.scanInput.requestFocus()

            }*/
            //Tarea 7751
            isBoxPickingInPrintOut.observe(viewLifecycleOwner) {

                if (it.isChecked){
                    ma.messageWithSound(
                        message = getString(R.string.errorBoxPicking),
                        isError = true,
                        isPlayed = true,
                        isToasted = false

                    )
                }else{
                    viewModel.updateExpeditionPrint(it.expeditionFk, expeditionPrintOut = ExpeditionPrintOut(it.expeditionFk,it.itemFk, true))

        }
                binding.scanInput.requestFocus()

            }
            responseCode.observe(viewLifecycleOwner) {

                ma.messageWithSound(
                    message = getString(R.string.errorInput),
                    isError = true,
                    isPlayed = true,
                    isToasted = null

                )
                binding.scanInput.requestFocus()

            }
        }
    }

}

