package es.verdnatura.presentation.view.feature.controlvehiculo.fragment

import android.content.Context
import android.graphics.drawable.Drawable
import android.util.TypedValue
import android.view.View.GONE
import android.view.inputmethod.EditorInfo
import android.widget.ImageView
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentVehiclecontrolBinding
import es.verdnatura.domain.toast
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnCollectionSelectedListener
import es.verdnatura.presentation.common.OnOptionsSelectedListener
import es.verdnatura.presentation.common.TAG
import es.verdnatura.presentation.common.ToolBarAdapterTooltip
import es.verdnatura.presentation.common.hideKeyboard
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogInput
import es.verdnatura.presentation.view.component.CustomDialogThreeButtons
import es.verdnatura.presentation.view.feature.historicovehiculo.fragment.HistoricoVehiculoFragment
import es.verdnatura.presentation.view.feature.pasillero.model.PasillerosItemVO
import es.verdnatura.presentation.view.feature.sacador.fragment.SacadorFragment

@Suppress("UNUSED_ANONYMOUS_PARAMETER")
class ControlVehiculoFragment(
    private var tagName: String = ""
) : BaseFragment<FragmentVehiclecontrolBinding, ControlVehiculoViewModel>(
    ControlVehiculoViewModel::class
) {

    private var goBack: Boolean = false
    private var onCollectionSelectedListener: OnCollectionSelectedListener? = null
    override fun getLayoutId(): Int = R.layout.fragment_vehiclecontrol
    private lateinit var customDialog: CustomDialog
    private var isWorkedTimed = "UNREGISTERED"
    private lateinit var customDialogInput: CustomDialogInput
    private lateinit var customDialogThreeButtons: CustomDialogThreeButtons

    companion object {
        fun newInstance(tagName: String) = ControlVehiculoFragment(tagName)
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        if (context is OnCollectionSelectedListener) onCollectionSelectedListener = context
    }

    override fun init() {

        customDialog = CustomDialog(requireContext())
        customDialogInput = CustomDialogInput(requireContext())
        customDialogThreeButtons = CustomDialogThreeButtons(requireContext())
        goBack = false
        if (tagName == HistoricoVehiculoFragment.TAG) {
            setToolBar("HISTORY")
            showScanner(getString(R.string.scanShelvingHistorical), "HISTORY")
        } else {

            checkControlTimeVehicle()
            setToolBar("ALL")
        }


        setEvents()
        super.init()
    }

    override fun onPause() {
        goBack = true
        super.onPause()
    }

    private fun setToolBar(direction: String?) {

        binding.mainToolbar.toolbarTitle.text = getString(R.string.vehiclecontrol)
        binding.mainToolbar.toolbarTitle.setTextSize(TypedValue.COMPLEX_UNIT_SP, 13F)

        val listIcons: ArrayList<ImageView> = ArrayList()
        val iconHistory = ImageView(context)
        iconHistory.setImageResource(R.drawable.ic_history_black_24dp)
        val iconVehicleIn = ImageView(context)
        iconVehicleIn.setImageResource(R.drawable.car)
        val iconVehicleOut = ImageView(context)
        iconVehicleOut.setImageResource(R.drawable.car_off)

        iconHistory.tooltipText = getTooltip(R.drawable.ic_history_black_24dp)
        iconVehicleIn.tooltipText = getTooltip(R.drawable.car)
        iconVehicleOut.tooltipText = getTooltip(R.drawable.car_off)

        when (direction) {
            "OUT" -> {
                listIcons.remove(iconVehicleIn)
                listIcons.add(iconVehicleOut)
                listIcons.add(iconHistory)
            }

            "IN" -> {
                listIcons.remove(iconVehicleOut)
                listIcons.add(iconVehicleIn)
                listIcons.add(iconHistory)
            }
        }


        binding.mainToolbar.toolbarIcons.adapter =
            ToolBarAdapterTooltip(listIcons, object : OnOptionsSelectedListener {
                override fun onOptionsItemSelected(item: Drawable) {

                    when (item) {
                        iconHistory.drawable -> showScanner(
                            getString(R.string.scanShelvingHistorical), "HISTORY"
                        )

                        iconVehicleIn.drawable -> showScanner(getString(R.string.scanPlate), "IN")
                        iconVehicleOut.drawable -> showScanner(
                            getString(R.string.scanPlateTakeoff), "OUT"
                        )
                    }
                }
            })
        binding.mainToolbar.toolbarIcons.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)
    }

    private fun showScanner(description: String, action: String) {

        customDialogInput.setTitle(getString(R.string.vehiclecontrol)).setDescription(description)
            .setOkButton(getString(R.string.accept)) {
                if (customDialogInput.getValue().isNotEmpty()) {
                    insertControlTimeVehicle(customDialogInput.getValue(), action)
                }
            }

            .setKoButton(getString(R.string.cancel)) {
                customDialogInput.dismiss()
                if (tagName == HistoricoVehiculoFragment.TAG) {
                    ma.onMyBackPressed()
                }

            }.setValue("").show()

        customDialogInput.setFocusText()

        customDialogInput.getEditText().setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {

                if (customDialogInput.getValue().isNotEmpty()) {
                    insertControlTimeVehicle(customDialogInput.getValue(), action)
                }
                return@setOnEditorActionListener true
            }
            false
        }

    }

    private fun checkControlTimeVehicle() {

        if (!goBack) {
            try {
                viewModel.workerMachineryIsRegistered(mobileApplication.userId!!)
            } catch (ex: Exception) {
                ma.messageWithSound(
                    message = getString(R.string.errorUserId),
                    isError = true,
                    isToasted = true,
                    isPlayed = true
                )
            }

        }
    }

    private fun insertControlTimeVehicle(plateNumber: String, direction: String) {

        when (direction) {
            "HISTORY" -> {
                ma.onPasillerosItemClickListener(
                    PasillerosItemVO(title = getString(R.string.titleHistoricalHide)), plateNumber
                )
            }

            "IN" -> {
                viewModel.machineWorkerUpdate(plateNumber)
            }

            "OUT" -> {
                viewModel.machineWorkerUpdate(plateNumber)
            }

            "" -> {
                ma.openFragmentPickers(true)
            }
        }

        customDialogInput.setValue("")
        scanRequest()
        customDialogInput.dismiss()
        hideKeyboards()
    }

    private fun scanRequest() {
        binding.scanInput.requestFocus()
        hideKeyboards()
    }

    private fun hideKeyboards() {
        try {
            requireActivity().hideKeyboard()
        } catch (e: Exception) {
            println(e.message)
        }
    }

    private fun setEvents() {

        binding.mainToolbar.backButton.setOnClickListener {
            ma.openFragmentPickers(true)

        }

        binding.scanInput.requestFocus()
        binding.scanInput.setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                if (!binding.scanInput.text.isNullOrEmpty()) {

                    if (tagName == HistoricoVehiculoFragment.TAG) {
                        showOptionsHistorical(binding.scanInput.text.toString())
                    } else {
                        showOptionsVehicleControl(binding.scanInput.text.toString())
                    }
                }
                binding.scanInput.setText("")
                ma.hideKeyboard(binding.scanInput)
                return@setOnEditorActionListener true
            }
            false
        }
    }

    private fun showOptionsVehicleControl(plate: String) {

        customDialogThreeButtons.setCustomDialogValue(GONE)

        customDialogThreeButtons.setDescription(getString(R.string.doVehiclePlate) + plate + "?")
            .setValue(plate)
        if (isWorkedTimed == "UNREGISTERED") {
            customDialogThreeButtons.setOkButtonTwo("COGER") {
                if (customDialogThreeButtons.getValue().trim().isEmpty()) {
                    getString(R.string.returnScan).toast(requireContext())
                } else {
                    customDialogThreeButtons.dismiss()
                    insertControlTimeVehicle(customDialogThreeButtons.getValue(), "IN")
                }
            }
        } else {

            customDialogThreeButtons.setOkButtonThree("DEJAR") {
                if (customDialogThreeButtons.getValue().trim().isEmpty()) {
                    getString(R.string.returnScan).toast(requireContext())
                } else {
                    customDialogThreeButtons.dismiss()
                    insertControlTimeVehicle(customDialogThreeButtons.getValue(), "OUT")
                }
            }
        }.setOkButtonFour(getString(R.string.historical)) {
            if (customDialogThreeButtons.getValue().trim().isEmpty()) {
                getString(R.string.returnScan).toast(requireContext())
            } else {
                customDialogThreeButtons.dismiss()
                insertControlTimeVehicle(customDialogThreeButtons.getValue(), "HISTORY")

            }

        }.setKoButton(getString(R.string.cancel)) {
            scanRequest()
            customDialogThreeButtons.dismiss()
        }.show()
    }

    private fun showOptionsHistorical(plate: String) {

        customDialogThreeButtons.setCustomDialogValue(GONE)

        customDialogThreeButtons.setDescription(getString(R.string.doVehiclePlate) + plate + "?")

            .setValue(plate)

            .setOkButtonFour(getString(R.string.historical)) {
                if (customDialogThreeButtons.getValue().trim().isEmpty()) {
                    getString(R.string.returnScan).toast(requireContext())
                } else {
                    customDialogThreeButtons.dismiss()
                    insertControlTimeVehicle(customDialogThreeButtons.getValue(), "HISTORY")

                }

            }.setKoButton(getString(R.string.cancel)) {
                scanRequest()
                customDialogThreeButtons.dismiss()
            }.show()
    }

    override fun observeViewModel() {

        with(viewModel) {
            responseUpdate.observe(viewLifecycleOwner) {
                ma.openFragmentPickers(false)
            }

            responseControl.observe(viewLifecycleOwner) {

                if (it == "1") {
                    isWorkedTimed = "REGISTERED"
                    setToolBar("OUT")

                    if (tagName != SacadorFragment.TAG) {
                        ma.openFragmentPickers(false)
                    }

                } else {

                    if (it == "0") {
                        setToolBar("IN")

                        customDialogInput.setTitle(getString(R.string.vehiclecontrol))
                            .setDescription(getString(R.string.scanPlate))
                            .setOkButton(getString(R.string.takeVehicle)) {
                                if (customDialogInput.getValue().isNotEmpty()) {
                                    insertControlTimeVehicle(customDialogInput.getValue(), "IN")
                                }
                                customDialogInput.setValue("")
                                scanRequest()
                                customDialogInput.dismiss()
                                hideKeyboards()
                            }

                            .setKoButton(getString(R.string.noVehicle)) {
                                customDialogInput.dismiss()
                                ma.openFragmentPickers(true)

                            }.setValue("").show()

                        customDialogInput.setFocusText()

                    }
                }
            }

        }

    }

}