package es.verdnatura.presentation.view.feature.paletizador.fragment

import android.view.View.GONE
import android.view.inputmethod.EditorInfo
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentExpeditionscanSorterBinding
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.GeneralAdapter
import es.verdnatura.presentation.common.GeneralItem
import es.verdnatura.presentation.common.OnGeneralItemRowClickListener
import es.verdnatura.presentation.view.component.CustomDialogList

class ExpeditionScanSorterFragment(
    var title: String = ""
) : BaseFragment<FragmentExpeditionscanSorterBinding, ExpeditionScanSorterViewModel>(
    ExpeditionScanSorterViewModel::class
) {

    override fun getLayoutId(): Int = R.layout.fragment_expeditionscan_sorter
    private lateinit var customDialogList: CustomDialogList
    private var listExpeditionScan: ArrayList<GeneralItem> = ArrayList()
    private var expeditionScanAdapter: GeneralAdapter? = null
    private var itemScaned: String = ""

    companion object {
        fun newInstance(title: String) = ExpeditionScanSorterFragment(title)
    }

    override fun init() {

        customDialogList = CustomDialogList(requireContext())
        ma.hideBottomNavigation(GONE)
        setEvents()
        setToolBar()
        showExpeditionScan()
        super.init()
    }

    private fun setToolBar() {
        binding.mainToolbar.toolbarTitle.text = title
    }

    private fun setEvents() {

        binding.mainToolbar.backButton.setOnClickListener {
            ma.onMyBackPressed()

        }
    }

    override fun observeViewModel() {

        with(viewModel) {
            responsescan.observe(viewLifecycleOwner) {
                listExpeditionScan.add(GeneralItem(itemScaned, itemScaned))
                customDialogList.setDescription(getString(R.string.total) + ":" + listExpeditionScan.size)
                expeditionScanAdapter!!.notifyItemInserted(listExpeditionScan.size - 1)
                customDialogList.setValue("")

            }

        }

    }

    private fun showExpeditionScan() {

        customDialogList.setTitle(getString(R.string.expeditionNoSorter))
            .setOkButton(getString(R.string.end)) {
                ma.hideKeyboard(customDialogList.getEditText())
                customDialogList.dismiss()
                binding.mainToolbar.backButton.performClick()
            }.setValue("").show()


        customDialogList.getEditText().requestFocus()
        ma.hideKeyboard(customDialogList.getEditText())
        customDialogList.getEditText().setOnEditorActionListener { _, actionId, _ ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {
                if (customDialogList.getValue().isNotEmpty()) {
                    viewModel.expeditionScan(customDialogList.getValue())
                    itemScaned = customDialogList.getValue()
                }
                customDialogList.setValue("")
                ma.hideKeyboard(customDialogList.getEditText())
                return@setOnEditorActionListener true
            }
            false
        }

        expeditionScanAdapter =
            GeneralAdapter(listExpeditionScan, object : OnGeneralItemRowClickListener {
                override fun OnGeneralItemRowClickListener(item: GeneralItem) {
                }
            })

        customDialogList.getRecyclerView().adapter = expeditionScanAdapter
        customDialogList.getRecyclerView().layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)

    }

}

