package es.verdnatura.presentation.view.feature.paletizador.fragment

import android.content.Context
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.map
import es.verdnatura.domain.SalixCallback
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.base.getMessageFromAllResponse
import es.verdnatura.presentation.base.nameofFunction
import es.verdnatura.presentation.common.Event
import es.verdnatura.presentation.common.ResponseItemVO
import es.verdnatura.presentation.view.feature.paletizador.model.ItemExpeditionScanList
import es.verdnatura.presentation.view.feature.paletizador.model.ItemExpeditionScanVO
import es.verdnatura.presentation.view.feature.paletizador.model.ItemScanList
import es.verdnatura.presentation.view.feature.paletizador.model.ItemScanVO
import retrofit2.Response

class ExpeditionScanViewModel(val context: Context) : BaseViewModel(context) {

    private val _expeditionScanList by lazy { MutableLiveData<ItemExpeditionScanList>() }
    val loadExpeditionScanList: LiveData<Event<ItemExpeditionScanList>> = _expeditionScanList.map { Event(it) }

    private val _scanList by lazy { MutableLiveData<ItemScanList>() }
    val loadScanList: LiveData<Event<ItemScanList>> = _scanList.map { Event(it) }

    private val _response by lazy { MutableLiveData<ResponseItemVO>() }
    val response: LiveData<ResponseItemVO>
        get() = _response

    private val _responseDeleteScan by lazy { MutableLiveData<ResponseItemVO>() }
    val responseDeleteScan: LiveData<ResponseItemVO>
        get() = _responseDeleteScan

    private val _responseCheckexpeditionScanPut by lazy { MutableLiveData<ResponseItemVO>() }
    val responseCheckexpeditionScanPut: LiveData<ResponseItemVO>
        get() = _responseCheckexpeditionScanPut

    fun expeditionScanList(vPalletFk: Int) {
        salix.expeditionScanList(arrayListOf(vPalletFk))
            .enqueue(object :
                SalixCallback<List<ItemExpeditionScanVO>>(context) {
                override fun onSuccess(response: Response<List<ItemExpeditionScanVO>>) {
                    if (response.body() != null) {
                        _expeditionScanList.value =
                            response.body()?.let { ItemExpeditionScanList(it) }
                    } else {
                        val listError: ArrayList<ItemExpeditionScanVO> = ArrayList()
                        listError.add(
                            ItemExpeditionScanVO(
                                0,
                                isError = true,
                                errorMessage = getMessageFromAllResponse(
                                    nameofFunction(this),
                                    response.message()
                                )
                            )
                        )
                        _expeditionScanList.value = ItemExpeditionScanList(listError)
                    }
                }

                override fun onError(t: Throwable) {
                    val listError: ArrayList<ItemExpeditionScanVO> = ArrayList()
                    listError.add(
                        ItemExpeditionScanVO(
                            0,
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                t.message!!
                            )
                        )
                    )
                    _expeditionScanList.value = ItemExpeditionScanList(listError)
                }

            })
    }

    fun expeditionScanAdd(vPalletFk: Int, vTruckFk: Int) {
        salix.expeditionScanAdd(arrayListOf(vPalletFk, vTruckFk))
            .enqueue(object :
                SalixCallback<List<ItemScanVO>>(context) {
                override fun onSuccess(response: Response<List<ItemScanVO>>) {
                    if (response.body() != null) {
                        _scanList.value = response.body()?.let { ItemScanList(it) }
                    } else {
                        val listError: ArrayList<ItemScanVO> = ArrayList()
                        listError.add(
                            ItemScanVO(
                                "", 0,
                                isError = true,
                                errorMessage = getMessageFromAllResponse(
                                    nameofFunction(this),
                                    response.message()
                                )
                            )
                        )
                        _scanList.value = ItemScanList(listError)
                    }
                }

                override fun onError(t: Throwable) {
                    val listError: ArrayList<ItemScanVO> = ArrayList()
                    listError.add(
                        ItemScanVO(
                            "", 0,
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                t.message!!
                            )
                        )
                    )
                    _scanList.value = ItemScanList(listError)
                }
            })
    }

    fun expeditionScanPut(
        vPalletFk: Int,
        vExpeditionFk: Int
    ) {
        salix.expeditionScanPut(
            arrayListOf(vPalletFk, vExpeditionFk)
        )
            .enqueue(object : SalixCallback<Any>(context) {
                override fun onSuccess(response: Response<Any>) {
                    _response.value =
                        ResponseItemVO(isError = false, response = response.message())

                }

                override fun onError(t: Throwable) {
                    _response.value = ResponseItemVO(
                        isError = true, errorMessage = getMessageFromAllResponse(
                            nameofFunction(this), t.message!!
                        )
                    )
                }
            })
    }

    fun checkRouteExpeditionScanPut(
        vPalletFk: Int,
        vExpeditionFk: Int
    ) {
        salix.expeditionCheckRoute(
            arrayListOf(
                vPalletFk,
                vExpeditionFk
            )
        ).enqueue(object : SalixCallback<String>(context) {
            override fun onSuccess(response: Response<String>) {
                if (response.body() == null) {
                    _responseCheckexpeditionScanPut.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(
                            nameofFunction(this),
                            response.message()
                        )
                    )
                } else {
                    _responseCheckexpeditionScanPut.value =
                        ResponseItemVO(isError = false, response = response.body()!!)

                }
            }

            override fun onError(t: Throwable) {
                _responseCheckexpeditionScanPut.value = ResponseItemVO(
                    isError = true,
                    errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                )
            }
        })
    }

    fun expeditionScanDel(vScanFk: Int) {
        salix.expeditionScanDel(vScanFk)
            .enqueue(object : SalixCallback<Any>(context) {
                override fun onError(t: Throwable) {
                    _responseDeleteScan.value = ResponseItemVO(
                        isError = true, errorMessage = getMessageFromAllResponse(
                            nameofFunction(this), t.message!!
                        )
                    )
                }

                override fun onSuccess(response: Response<Any>) {

                    _responseDeleteScan.value =
                        ResponseItemVO(isError = false, response = response.message())
                }
            })
    }

}
